using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace FactoryExample
{
	public class BulletsObjectsPoolProxy : BulletFactory, IFactory<BulletBehaviour>
	{
		[SerializeField]
		private BulletFactory _factory;

		[SerializeField]
		private int _preloadCount = 10;

		private Queue<BulletBehaviour> _bullets = new Queue<BulletBehaviour>();

		private void Start()
		{
			for (var i = 0; i < _preloadCount; i++)
			{
				var obj = _factory.Create();

				obj.BulletsObjectsPool = this;

				AddToPool(obj);
			}
		}

		public override BulletBehaviour Create()
		{
			if (_bullets.Count > 0)
			{
				var bullet = _bullets.Dequeue();
				bullet.gameObject.SetActive(true);

				return bullet;
			}
			else
			{
				var obj = _factory.Create();
				obj.transform.SetParent(transform, true);

				obj.BulletsObjectsPool = this;

				return obj;
			}
		}

		public void AddToPool(BulletBehaviour bullet)
		{
			bullet.transform.parent = transform;
			bullet.gameObject.SetActive(false);

			_bullets.Enqueue(bullet);
		}
	}
}
