using System;
using System.Collections.Generic;

public abstract class Signal
{
	private static Dictionary<Type, List<Action>> _signals = new Dictionary<Type, List<Action>>();

	public void AddListener(Action act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			_signals.Add(type, new List<Action>());
		}

		_signals[type].Add(act);
	}

	public void RemoveListener(Action act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		_signals[type].Remove(act);
	}

	public void Dispatch()
	{
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		foreach (var act in _signals[type])
		{
			act?.Invoke();
		}
	}
}

public abstract class Signal<T>
{
	private static Dictionary<Type, List<Action<T>>> _signals = new Dictionary<Type, List<Action<T>>>();

	public void AddListener(Action<T> act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			_signals.Add(type, new List<Action<T>>());
		}

		_signals[type].Add(act);
	}

	public void RemoveListener(Action<T> act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		_signals[type].Remove(act);
	}

	public void Dispatch(T data)
	{
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		foreach (var act in _signals[type])
		{
			act?.Invoke(data);
		}
	}
}

public abstract class Signal<T1, T2>
{
	private static Dictionary<Type, List<Action<T1, T2>>> _signals = new Dictionary<Type, List<Action<T1, T2>>>();

	public void AddListener(Action<T1, T2> act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			_signals.Add(type, new List<Action<T1, T2>>());
		}

		_signals[type].Add(act);
	}

	public void RemoveListener(Action<T1, T2> act)
	{
		if (act == null)
		{
			return;
		}
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		_signals[type].Remove(act);
	}

	public void Dispatch(T1 data1, T2 data2)
	{
		var type = this.GetType();

		if (!_signals.ContainsKey(type))
		{
			return;
		}

		foreach (var act in _signals[type])
		{
			act?.Invoke(data1, data2);
		}
	}
}